#!/usr/bin/python
#
#
# whiteboard.py - Wiimote whiteboard tracking software 
# 
#	REQUIRED LIBRARIES: python-xlib, pybluez, (psyco: optional)
#
#		Other files: "linuxWiimoteLib.py", "mousecontrol.py" 
#				"keypressreader" and "perspective.py"
#
#	TROUBLESHOOTING:
#
#	If you get errors about bluetooth, make sure you have pybluez installed,
#	as well as pybluez-utils. then, restart your bluetooth service 
#	(/etc/init.d/bluetooth restart) or your computer.
#
#	Tested and working on Mandriva 2007.1
#
#
# Written by Stephane Duchesneau <stephane.duchesneau@gmail.com>
# 
# 


try:
	import psyco
	psyco.full()
except:
	print "Not using psyco"

import time
from mousecontrol import MouseControl
from linuxWiimoteLib import WiimoteLib
from perspective import Perspective
from keypressreader import KeypressReader

class WhiteBoard():
	running = False
	mouseclicked = False
	calibrating = False	
	def __init__(self):
		self.mouse = MouseControl()
		self.XRES,self.YRES = self.mouse.get_screen_resolution()
		self.XRES, self.YRES = float(self.XRES), float(self.YRES)
		self.ABCD = [(0,0),(self.XRES-5,0),(0,self.YRES-5),(self.XRES-5,self.YRES-5)]
		self.wiimote = WiimoteLib()
		self.wiimote.Connect()
		self.wiimote.SetLEDs(True,False,False,False)
		print "Battery: ", self.wiimote.Battery, "%"
		self.wiimote.activate_accel()
		self.wiimote.activate_IR()
		self.perspective = Perspective()
		self.perspective.setdst((0.0,0.0),(self.XRES,0.0),(0.0,self.YRES),(self.XRES,self.YRES))
		self.keypressreader = KeypressReader()
	
	def run(self):
		
		self.running = True
		self.counterclic = 0
		self.countersingleclic = 0
		self.counterdoubleclic = 0
		last_x,last_y = self.wiimote.IRState.RawX1, self.wiimote.IRState.RawY1
		while self.running:
			time.sleep(0.030)
			
			x,y =  self.wiimote.IRState.RawX1, self.wiimote.IRState.RawY1
			
			if self.countersingleclic: self.countersingleclic -= 1
			
			if not ((x == 1023) and (y == 1023)) :

				x,y = self.perspective.warp(x,y)
				#print x, " ",y
				if ((x<0) or (x>self.XRES)): # The light spot is on either sides of the screen field: Right click
					self.mouse.mouse_down(3)  	# Right clic, left border or right border
					time.sleep(0.2)
					self.mouse.mouse_up(3)
					x = last_x			
					y = last_y
					self.counterclic=0
				
				else: # We see a light spot in the screen field: either move or move and click
					if not self.countersingleclic: self.mouse.mouse_warp(x,y)
					if not self.mouseclicked:
						self.mouse.mouse_down(1)
						self.mouseclicked = True
						self.counterclic = 5 # delay for mouse_up state, safe for draw&drop
						self.countersingleclic = 4 # delay for moving after click, safe for single-clic !
						last_x = x
						last_y = y

			else: #no light
				if self.mouseclicked == True:
					self.counterclic-=1
					if self.counterclic <= 0:
						self.mouse.mouse_up(1)
						self.mouseclicked = False
			self.handle_events()
			
	
	def stop(self):
		self.running = False
		self.keypressreader.stop()
		self.wiimote.Disconnect()
		if self.calibrating: raise SystemExit
		
		
	def handle_events(self, wiibuttons = True):#wiibuttons=false means only check keyboard
		key = self.keypressreader.check_for_keypress()
		if key == "q": self.stop()
		elif key == "c" or (self.wiimote.ButtonState.A and wiibuttons): self.calibrate()

		
		
	def calibrate(self):
		self.calibrating = True
		print "Point to the top left corner of the screen"
		self.ABCD[0] = self.wait_for_light_click()
		
		self.wait_for_no_light()
		print "Point to the top right corner of the screen"
		self.ABCD[1] = self.wait_for_light_click()
		
		self.wait_for_no_light()
		print "Point to the bottom left corner of the screen"
		self.ABCD[2] = self.wait_for_light_click()
		
		self.wait_for_no_light()
		print "Point to the bottom right corner of the screen"
		self.ABCD[3] = self.wait_for_light_click()
		
		self.wait_for_no_light()
		x0,y0 = self.ABCD[0]
		x1,y1 = self.ABCD[1]
		x2,y2 = self.ABCD[2]
		x3,y3 = self.ABCD[3]
		self.perspective.setsrc((x0,y0),(x1,y1),(x2,y2),(x3,y3))
		
		self.calibrating = False
		print "Calibration completed ! Enjoy !"
		print ""
		print "Press 'q' to quit."
		print "Press 'c' or (A) on the wiimote to re-calibrate"
		
	def wait_for_light_click(self):
		dot = False
		while not dot:
			checkdot = self.wiimote.IRState.RawX1, self.wiimote.IRState.RawY1
			if self.wiimote.IRState.Found1:
				dot = checkdot
				print dot
				return dot
			time.sleep(0.030)
			self.handle_events(wiibuttons=False)
			
	def wait_for_no_light(self):
		time.sleep(0.5)
		while 1:
			time.sleep(0.030)
			if self.wiimote.IRState.Found1 == False:
				return
			self.handle_events(wiibuttons=False)



whiteboard = WhiteBoard()
whiteboard.calibrate()
try:
	whiteboard.run()
finally:
	whiteboard.keypressreader.stop()
